@@ -51,6 +51,7 @@ My Section Bundle Content Content.
 title: "My Section"
 ---
 -- content/mysection/mysectiontext.txt --
+Content.
 -- content/_index.md --
 ---
 title: "Home"
@@ -99,15 +100,17 @@ My Other Text: {{ $r.Content }}|{{ $r.Permalink }}|
 `
 
 func TestRebuildEditLeafBundleHeaderOnly(t *testing.T) {
-	b := TestRunning(t, rebuildFilesSimple)
-	b.AssertFileContent("public/mysection/mysectionbundle/index.html",
-		"My Section Bundle Content Content.")
-
-	b.EditFileReplaceAll("content/mysection/mysectionbundle/index.md", "My Section Bundle Content.", "My Section Bundle Content Edited.").Build()
-	b.AssertFileContent("public/mysection/mysectionbundle/index.html",
-		"My Section Bundle Content Edited.")
-	b.AssertRenderCountPage(2) // home (rss) + bundle.
-	b.AssertRenderCountContent(1)
+	t.Parallel()
+	for i := 0; i < 3; i++ {
+		b := TestRunning(t, rebuildFilesSimple)
+		b.AssertFileContent("public/mysection/mysectionbundle/index.html",
+			"My Section Bundle Content Content.")
+		b.EditFileReplaceAll("content/mysection/mysectionbundle/index.md", "My Section Bundle Content.", "My Section Bundle Content Edited.").Build()
+		b.AssertFileContent("public/mysection/mysectionbundle/index.html",
+			"My Section Bundle Content Edited.")
+		b.AssertRenderCountPage(2) // home (rss) + bundle.
+		b.AssertRenderCountContent(1)
+	}
 }
 
 func TestRebuildEditTextFileInLeafBundle(t *testing.T) {
@@ -119,12 +122,12 @@ func TestRebuildEditTextFileInLeafBundle(t *testing.T) {
 	b.AssertFileContent("public/mysection/mysectionbundle/index.html",
 		"Text 2 Content Edited")
 	b.AssertRenderCountPage(1)
-	b.AssertRenderCountContent(1)
+	b.AssertRenderCountContent(0)
 }
 
 func TestRebuildEditTextFileInShortcode(t *testing.T) {
 	t.Parallel()
-	for i := 0; i < 3; i++ {
+	for range 3 {
 		b := TestRunning(t, rebuildFilesSimple)
 		b.AssertFileContent("public/mythirdsection/mythirdsectionpage/index.html",
 			"Text: Assets My Shortcode Text.")
@@ -138,7 +141,7 @@ func TestRebuildEditTextFileInShortcode(t *testing.T) {
 
 func TestRebuildEditTextFileInHook(t *testing.T) {
 	t.Parallel()
-	for i := 0; i < 3; i++ {
+	for range 3 {
 		b := TestRunning(t, rebuildFilesSimple)
 		b.AssertFileContent("public/mythirdsection/mythirdsectionpage/index.html",
 			"Text: Assets My Other Text.")
@@ -180,17 +183,17 @@ func TestRebuildEditTextFileInHomeBundle(t *testing.T) {
 	b.AssertFileContent("public/index.html", "Home Content.")
 	b.AssertFileContent("public/index.html", "Home Text Content Edited.")
 	b.AssertRenderCountPage(1)
-	b.AssertRenderCountContent(1)
+	b.AssertRenderCountContent(0)
 }
 
 func TestRebuildEditTextFileInBranchBundle(t *testing.T) {
 	b := TestRunning(t, rebuildFilesSimple)
-	b.AssertFileContent("public/mysection/index.html", "My Section")
+	b.AssertFileContent("public/mysection/index.html", "My Section", "0:/mysection/mysectiontext.txt|Content.|")
 
 	b.EditFileReplaceAll("content/mysection/mysectiontext.txt", "Content.", "Content Edited.").Build()
-	b.AssertFileContent("public/mysection/index.html", "My Section")
+	b.AssertFileContent("public/mysection/index.html", "My Section", "0:/mysection/mysectiontext.txt|Content Edited.|")
 	b.AssertRenderCountPage(1)
-	b.AssertRenderCountContent(1)
+	b.AssertRenderCountContent(0)
 }
 
 func testRebuildBothWatchingAndRunning(t *testing.T, files string, withB func(b *IntegrationTestBuilder)) {
@@ -208,7 +211,7 @@ func TestRebuildRenameTextFileInLeafBundle(t *testing.T) {
 		b.RenameFile("content/mysection/mysectionbundle/mysectionbundletext.txt", "content/mysection/mysectionbundle/mysectionbundletext2.txt").Build()
 		b.AssertFileContent("public/mysection/mysectionbundle/index.html", "mysectionbundletext2", "My Section Bundle Text 2 Content.", "Len Resources: 2|")
 		b.AssertRenderCountPage(8)
-		b.AssertRenderCountContent(8)
+		b.AssertRenderCountContent(9)
 	})
 }
 
@@ -357,8 +360,8 @@ RegularPages: {{ range .Site.RegularPages }}{{ .RelPermalink }}|{{ end }}$
 }
 
 func TestRebuildRenameDirectoryWithBranchBundleFastRender(t *testing.T) {
-	recentlyVisited := types.NewEvictingStringQueue(10).Add("/a/b/c/")
-	b := TestRunning(t, rebuildFilesSimple, func(cfg *IntegrationTestConfig) { cfg.BuildCfg = BuildCfg{RecentlyVisited: recentlyVisited} })
+	recentlyVisited := types.NewEvictingQueue[string](10).Add("/a/b/c/")
+	b := TestRunning(t, rebuildFilesSimple, func(cfg *IntegrationTestConfig) { cfg.BuildCfg = BuildCfg{RecentlyTouched: recentlyVisited} })
 	b.RenameDir("content/mysection", "content/mysectionrenamed").Build()
 	b.AssertFileContent("public/mysectionrenamed/index.html", "My Section")
 	b.AssertFileContent("public/mysectionrenamed/mysectionbundle/index.html", "My Section Bundle")
@@ -484,7 +487,43 @@ Home: {{ .Title }}|{{ .Content }}|
 	})
 }
 
-func TestRebuildSingleWithBaseof(t *testing.T) {
+func TestRebuildSingle(t *testing.T) {
+	t.Parallel()
+
+	files := `
+-- hugo.toml --
+title = "Hugo Site"
+baseURL = "https://example.com"
+disableKinds = ["term", "taxonomy", "sitemap", "robotstxt", "404"]
+disableLiveReload = true
+-- content/p1.md --
+---
+title: "P1"
+---
+P1 Content.
+-- layouts/index.html --
+Home.
+-- layouts/single.html --
+Single: {{ .Title }}|{{ .Content }}|
+{{ with (templates.Defer (dict "key" "global")) }}
+Defer.
+{{ end }}
+`
+	b := Test(t, files, TestOptRunning())
+	b.AssertFileContent("public/p1/index.html", "Single: P1|", "Defer.")
+	b.AssertRenderCountPage(3)
+	b.AssertRenderCountContent(1)
+	b.EditFileReplaceFunc("layouts/single.html", func(s string) string {
+		s = strings.Replace(s, "Single", "Single Edited", 1)
+		s = strings.Replace(s, "Defer.", "Defer Edited.", 1)
+		return s
+	}).Build()
+	b.AssertFileContent("public/p1/index.html", "Single Edited: P1|", "Defer Edited.")
+	b.AssertRenderCountPage(1)
+	b.AssertRenderCountContent(0)
+}
+
+func TestRebuildSingleWithBaseofEditSingle(t *testing.T) {
 	t.Parallel()
 
 	files := `
@@ -498,9 +537,13 @@ disableLiveReload = true
 title: "P1"
 ---
 P1 Content.
+[foo](/foo)
 -- layouts/_default/baseof.html --
 Baseof: {{ .Title }}|
 {{ block "main" . }}default{{ end }}
+{{ with (templates.Defer (dict "foo" "bar")) }}
+Defer.
+{{ end }}
 -- layouts/index.html --
 Home.
 -- layouts/_default/single.html --
@@ -509,11 +552,81 @@ Single: {{ .Title }}|{{ .Content }}|
 {{ end }}
 `
 	b := Test(t, files, TestOptRunning())
-	b.AssertFileContent("public/p1/index.html", "Baseof: P1|\n\nSingle: P1|<p>P1 Content.</p>\n|")
+	b.AssertFileContent("public/p1/index.html", "Single: P1|")
 	b.EditFileReplaceFunc("layouts/_default/single.html", func(s string) string {
 		return strings.Replace(s, "Single", "Single Edited", 1)
 	}).Build()
-	b.AssertFileContent("public/p1/index.html", "Baseof: P1|\n\nSingle Edited: P1|<p>P1 Content.</p>\n|")
+	b.AssertFileContent("public/p1/index.html", "Single Edited")
+}
+
+func TestRebuildSingleWithBaseofEditBaseof(t *testing.T) {
+	t.Parallel()
+
+	files := `
+-- hugo.toml --
+title = "Hugo Site"
+baseURL = "https://example.com"
+disableKinds = ["term", "taxonomy"]
+disableLiveReload = true
+-- content/p1.md --
+---
+title: "P1"
+---
+P1 Content.
+[foo](/foo)
+-- layouts/_default/baseof.html --
+Baseof: {{ .Title }}|
+{{ block "main" . }}default{{ end }}
+{{ with (templates.Defer (dict "foo" "bar")) }}
+Defer.
+{{ end }}
+-- layouts/index.html --
+Home.
+-- layouts/_default/single.html --
+{{ define "main" }}
+Single: {{ .Title }}|{{ .Content }}|
+{{ end }}
+`
+	b := Test(t, files, TestOptRunning())
+	b.AssertFileContent("public/p1/index.html", "Single: P1|")
+	fmt.Println("===============")
+	b.EditFileReplaceAll("layouts/_default/baseof.html", "Baseof", "Baseof Edited").Build()
+	b.AssertFileContent("public/p1/index.html", "Baseof Edited")
+}
+
+func TestRebuildWithDeferEditRenderHook(t *testing.T) {
+	t.Parallel()
+
+	files := `
+-- hugo.toml --
+title = "Hugo Site"
+baseURL = "https://example.com"
+disableKinds = ["term", "taxonomy"]
+disableLiveReload = true
+-- content/p1.md --
+---
+title: "P1"
+---
+P1 Content.
+[foo](/foo)
+-- layouts/_default/baseof.html --
+Baseof: {{ .Title }}|
+{{ block "main" . }}default{{ end }}
+ {{ with (templates.Defer (dict "foo" "bar")) }}
+Defer.
+{{ end }}
+-- layouts/single.html --
+{{ define "main" }}
+Single: {{ .Title }}|{{ .Content }}|
+{{ end }}
+-- layouts/_default/_markup/render-link.html --
+Render Link.
+`
+	b := Test(t, files, TestOptRunning())
+	// Edit render hook.
+	b.EditFileReplaceAll("layouts/_default/_markup/render-link.html", "Render Link", "Render Link Edited").Build()
+
+	b.AssertFileContent("public/p1/index.html", "Render Link Edited")
 }
 
 func TestRebuildFromString(t *testing.T) {
@@ -1181,6 +1294,49 @@ Content: {{ .Content }}
 	b.AssertFileContent("public/index.html", "Content: <p>Home</p>")
 }
 
+// Issue #13014.
+func TestRebuildEditNotPermalinkableCustomOutputFormatTemplateInFastRenderMode(t *testing.T) {
+	t.Parallel()
+
+	files := `
+-- hugo.toml --
+baseURL = "https://example.com/docs/"
+disableLiveReload = true
+[internal]
+fastRenderMode = true
+disableKinds = ["taxonomy", "term", "sitemap", "robotsTXT", "404"]
+[outputFormats]
+  [outputFormats.SearchIndex]
+    baseName = 'Search'
+    isPlainText = true
+    mediaType = 'text/plain'
+    noAlternative = true
+	permalinkable = false
+
+[outputs]
+  home = ['HTML', 'SearchIndex']
+-- content/_index.md --
+---
+title: "Home"
+---
+Home.
+-- layouts/index.html --
+Home.
+-- layouts/_default/index.searchindex.txt --
+Text. {{ .Title }}|{{ .RelPermalink }}|
+
+`
+	b := TestRunning(t, files, TestOptInfo())
+
+	b.AssertFileContent("public/search.txt", "Text.")
+
+	b.EditFileReplaceAll("layouts/_default/index.searchindex.txt", "Text.", "Text Edited.").Build()
+
+	b.BuildPartial("/docs/search.txt")
+
+	b.AssertFileContent("public/search.txt", "Text Edited.")
+}
+
 func TestRebuildVariationsAssetsJSImport(t *testing.T) {
 	t.Parallel()
 	files := `
@@ -1502,7 +1658,7 @@ title: "P%d"
 P%d Content.
 `
 
-	for i := 0; i < count; i++ {
+	for i := range count {
 		files += fmt.Sprintf("-- content/mysect/p%d/index.md --\n%s", i, fmt.Sprintf(contentTemplate, i, i))
 	}
 
@@ -1709,3 +1865,30 @@ p1-content|
 	b.EditFileReplaceAll("content/p1/index.md", "p1-content", "p1-content-foo").Build()
 	b.AssertFileContent("public/p1/index.html", "p1-content-foo")
 }
+
+func TestRebuildEditTagIssue13648(t *testing.T) {
+	t.Parallel()
+
+	files := `
+-- hugo.toml --
+baseURL = "https://example.com"
+disableLiveReload = true
+-- layouts/all.html --
+All. {{ range .Pages }}{{ .Title }}|{{ end }}
+-- content/p1.md --
+---
+title: "P1"
+tags: ["tag1"]
+---
+
+`
+	b := TestRunning(t, files)
+
+	b.AssertFileContent("public/tags/index.html", "All. Tag1|")
+	b.EditFileReplaceAll("content/p1.md", "tag1", "tag2").Build()
+
+	// Note that the below is still not correct, as this is effectively a rename, and
+	// Tag2 should be removed from the list.
+	// But that is a harder problem to tackle.
+	b.AssertFileContent("public/tags/index.html", "All. Tag1|Tag2|")
+}
